PROGRAM srbo
  USE kinds, ONLY: wp => dp
  USE constants, ONLY: pi, bohr
  USE bos

  IMPLICIT NONE

  REAL (KIND=wp) :: re, de, ke, al, v
  REAL (KIND=wp) :: vfact, vthrs
  INTEGER :: i, j, k, n, ma, mr
  REAL (KIND=wp) :: f
  REAL (KIND=wp) :: r, dr
  REAL (KIND=wp) :: dn, edge
  REAL (KIND=wp) :: rmin, rmax, be
  REAL (KIND=wp) :: bei, bef, dbe
  REAL (KIND=wp) :: th, thi, thf, dth
  INTEGER :: nbe, nth
  TYPE (bo) :: boab
  CHARACTER (LEN=72) :: infile, outfile
  LOGICAL :: filefound, betafound
  REAL (KIND=wp), DIMENSION(:), ALLOCATABLE :: nab
  CHARACTER (LEN=5) :: n_str

  ! h2: values from marston89_3571
  ! re = 1.40201_wp
  ! de = 0.1744_wp
  ! beta = 1.02764_wp

  ! get input filename from argument 1
  CALL getarg ( 1, infile )
  IF ( infile == "                                                                        ") THEN
    PRINT *
    PRINT *, "No input file (command line argument specified). Usage is:"
    PRINT *
    PRINT *, "   srbo [infile]"
    PRINT *
    STOP
  ELSE
    INQUIRE ( FILE = infile, EXIST = filefound )
    IF (.NOT.filefound) THEN
      PRINT *, "Input file ", TRIM(infile)," not found. Exit"
      STOP
    ENDIF
  ENDIF

  OPEN (UNIT=5, FILE=TRIM(infile), ACTION="read", STATUS="old")

  ! system data
  READ (UNIT=5, FMT=*) re, de, ke
  ! edges criteria
  READ (UNIT=5, FMT=*) vfact, vthrs
  ! grid data
  READ (UNIT=5, FMT=*) mr, ma

  CLOSE (5)

  al = SQRT ( ke / (2.0_wp * de) )

  PRINT *
  PRINT *, "SRBO program."
  PRINT *, "Sergio Rampino, Perugia 2015"

  PRINT *
  PRINT *, "System info: "
  PRINT "(A,F9.4)", "   re:      ", re
  PRINT "(A,F9.4)", "   de:      ", de
  PRINT "(A,F9.4)", "   ke:      ", ke
  PRINT "(A,F9.4)", "   alpha:   ", al

  PRINT *
  PRINT *, "Boundaries: "

  ! find rmin
  v = vfact * de
  rmin = re - LOG ( 1.0_wp + SQRT (v/de) ) / al

  PRINT "(A,F9.4)", "   rmin:    ", rmin
  !PRINT *, "V(rmin): ", de * ( 1.0_wp - EXP(-al*(rmin-re)) )**2.0
  !PRINT *, "de from V(rmin): ", de * ( 1.0_wp - EXP(-al*(rmin-re)) )**2.0 / vfact
  !PRINT *, de

  ! find rmax
  v = (1.0_wp - vthrs) * de
  rmax = re - LOG(1.0_wp - SQRT (v/de) ) / al

  PRINT "(A,F9.4)", "   rmax:    ", rmax
  !PRINT *, "V(rmax): ", de * ( 1.0_wp - EXP(-al*(rmax-re)) )**2.0
  !PRINT *, "de from V(rmax): ", de * ( 1.0_wp - EXP(-al*(rmax-re)) )**2.0 / (1.0_wp - vthrs)
  !PRINT *, de

! set f
  f = ma/REAL(mr)

  !n = NINT(m1 * (1 + f)) + 1

  n = ma + mr + 1

  PRINT *
  PRINT *, "Beta: "

  ! work out beta
  bei = 0.001_wp
  bef = 2.0_wp
  dbe = 0.00001_wp

  nbe = (bef - bei)/dbe + 1

  ROOTFIND: DO i = 1, nbe
    be = bei + (i -1) * dbe

    !PRINT "(I5, 2F13.5)", i, be, (EXP(-be*(rmin-re))-1.0_wp)/(1.0_wp-EXP(-be*(rmax-re)))

    IF ( ( (1.0_wp-EXP(-be*(rmax-re))) / (EXP(-be*(rmin-re))-1.0_wp) - f ) < 1.0E-6_wp ) THEN
      !PRINT *, "Found:", be
      betafound = .TRUE.
      EXIT ROOTFIND
    ENDIF


  ENDDO ROOTFIND

  IF ( .NOT. betafound) THEN
    PRINT *, "beta not found. see source for searching criteria..."
    STOP
  ENDIF

  PRINT "(A,F9.4)", "   Found:   ", be


  PRINT *
  PRINT *, "Setting up (", mr, ",", ma, ") grid: ", n, "points"
  
  ALLOCATE ( nab(n) )

  CALL bo_set ( boab, be, re)

  PRINT "(A,F9.4)", "   nmin:    ", bo_eval (boab, rmax)
  PRINT "(A,F9.4)", "   nmax:    ", bo_eval (boab, rmin)

  CALL bo_get_edggrid ( boab, nab, rmin, rmax )

  !PRINT *
  !DO i = 1, n
  !  PRINT "(2F15.7)", bo_eval_bl (boab, nab(i)), morse ( bo_eval_bl (boab, nab(i)), de, re, al )
  !ENDDO

  PRINT *
  PRINT *, "SRBO grid:"
  PRINT *

  WRITE (UNIT=n_str, FMT="(I5)") n

  WRITE (UNIT=6, FMT="(A,"//TRIM(n_str)//"F10.4,A)") "gr=[", (bo_eval_bl (boab, nab(i)), i=n,1,-1), "]"
  
  PRINT *
  PRINT *, "BL grid:"
  PRINT *

  dr = (rmax - rmin) / (n - 1)

  WRITE (UNIT=6, FMT="(A,"//TRIM(n_str)//"F10.4,A)") "gr=[", (rmin + (i-1)*dr, i=1,n), "]"

  DEALLOCATE ( nab )

END PROGRAM srbo
